#region usings
using System;
using System.Net;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel.Composition;
using VVVV.Hosting;
using System.Linq;
using VVVV.PluginInterfaces.V1;
using VVVV.PluginInterfaces.V2;
using VVVV.PluginInterfaces.V2.Graph;
using VVVV.Utils.OSC;
using VVVV.Core.Logging;
using VVVV.Core;
using System.Xml;

#endregion usings

namespace VVVV.Nodes
{
	#region PluginInfo
	[PluginInfo(Name = "Enable", Category = "vvvv", Help = "Enable/Disable given patch", Tags = "vvvv", AutoEvaluate = true)]
	#endregion PluginInfo
	public class ValueDisableNode : IPluginEvaluate
	{
		#region fields & pins
		[Input("Enabled", DefaultValue = 1, IsSingle=true)]
		IDiffSpread<bool> FInput;
		
		[Input("PatchID", DefaultValue = -1,IsSingle=true)]
		ISpread<int>FPatchID;
		
		[Output("Affected Nodes", Order= 1)]
		ISpread<string> FNodes;
		
		[Output("Disabled", DefaultValue = 0, IsSingle=true, Order= 2)]
		ISpread<bool> FDisable;
		
		[Import()]
		ILogger FLogger;
		
		[Import]
		protected NodeCollection FNodeCollection;
		
		[Import]
		protected INodeInfoFactory FNodesInfoFactory;
		
		[Import]
		IHDEHost FHDEHost;

		[ImportingConstructor]
		public ValueDisableNode(IHDEHost host)
		{
			FHDEHost = host;
		}
		#endregion fields & pins

		//called when data for any output pin is requested
		public void Evaluate(int SpreadMax)
		{
    		if(FInput.IsChanged) 
    		{
    			var PatchID = FPatchID[0];
				var result = FHDEHost.RootNode[0];
				string XMLmsg = "";
    			
    			
    			
    			
    			FNodes.SliceCount = 0;
    			
    			//get parent patch object, is there a better way?
    			var thisPatch = (from node in result where node.ID == PatchID select node);
    			
    			if(thisPatch.Count()>0) //did we found our parent?
    			{
	    			//get all enable-able nodes, getChildren returns INode!
	    			var patch = thisPatch.First(); 
	    			var enableables = (from node in patch.InternalCOMInterf.GetChildren() where (node.GetPin("Enabled") != null) select node.GetNodePath(false));
		            
    				//FLogger.Log(LogType.Debug, "___________________________________");
    				
    				
	    			//send message to pin(s) Enable
	    			if (enableables != null)
		            {
		            	FNodes.SliceCount = enableables.Count();
		            	int i = 0;
		           		
		            	XMLmsg = "";
		            	PatchMessage pm = new PatchMessage(patch.NodeInfo.Filename);
		                		
		                foreach (var nodePath in enableables)
		                {
		                	var node = FHDEHost.GetNodeFromPath(nodePath); //INode2
		                	if(node == null) 
		                	{
		                		FLogger.Log(LogType.Debug, nodePath + " not found");
		                		continue;
		                	}
		                	
		                	var pin = node.FindPin("Enabled"); //IPin2
		                	
		                	if(pin!=null) if(pin.ConnectedPins.Count == 0)
		                	{
		                		//Strategy1: set directly, does not update correctly?
	    						//pin.Spread = (FInput[0]) ? "1" : "0"; 
		                		
		                		//Strategy2: create patch msg (or bundle?), add null checks?
		                		//this returns error!? PatchClass "NNN" not found!
		                		
		                		var _node = pm.AddNode(node.ID);
								var _pin = _node.AddPin("Enabled");
								_pin.Spread = (FInput[0]) ? "1" : "0";
	
		                		//Strategy3: build XML for SetPatch Node
		                		XMLmsg += string.Format("<NODE id=\"{0}\"><PIN pinname=\"Enabled\" values=\"{1}\"></PIN></NODE>",node.ID.ToString(), (FInput[0]) ? "1" : "0");
		            			
		                		//report back which nodes were affected
		                		FNodes[i] = node.Name;
		                		i++;
		                		
		                	}
		                	
		                }
	
		            	//FHDEHost.SendXMLSnippet(patch.NodeInfo.Filename,  "<PATCH>" + XMLmsg + "</PATCH>", true);
		             	FHDEHost.SendXMLSnippet(patch.NodeInfo.Filename, pm.ToString(), true);
				        FDisable[0] = !FInput[0];
		            
		            	if( i < FNodes.SliceCount) FNodes.SliceCount = i; //correct the SliceCount, must be always less or equ
		           		
	    			}
    				
    			}
    		} 
    		 
			//FLogger.Log(LogType.Debug, "hi tty!");
		}
	}
}
